const express = require("express");
const router = express.Router();
const catchAsync = require('../utils/catchAsync');
const ExpressError = require('../utils/ExpressError.js');
const { isLoggedIn, isStudent } = require('../middleware');

const Ispit = require('../models/ispit');
const Student = require('../models/student');
const Kolegij = require('../models/kolegij');


//putanje se odnose na profil jednog studenta
//dohvat podataka jednog studenta, po ideji prijavljenog u sustav
router.get('/', isLoggedIn, isStudent, catchAsync(async (req, res) => {
    const username = req.user.username;

    const student = await Student.findOne({username: username}).populate('kolegiji').populate('ispiti');
    res.render('student/index', { student });

}))

//dohvat stranice ispita
router.get('/ispit/:id/:studentId', isLoggedIn, isStudent, catchAsync(async (req, res) => {
    const {id, studentId} = req.params;
    const ispit = await Ispit.findById(id).populate('kolegij');
    let ocjena = null;
    for(let r of ispit.rezultati){
        if(r.student.equals(studentId))ocjena = r.ocjena;
    }

    res.render('student/ispit', {ispit, ocjena, studentId})
}))

//odjava ispita
router.put('/ispit/:studentId/:ispitId', isLoggedIn, isStudent, catchAsync(async (req, res) =>{
    const {studentId, ispitId} = req.params;
    const student = await Student.findById(studentId);
    const ispit = await Ispit.findById(ispitId);
    const rezultati = ispit.rezultati.filter(rez => !(rez.student.equals(studentId)))
    ispit.rezultati = rezultati;
    await ispit.save();
    const ispiti = student.ispiti.filter(ispit => !(ispit._id.equals(ispitId)))
    student.ispiti = ispiti;
    await student.save();
    
    res.redirect('/student');

}))

//putanja za dohvat podataka o kolegiju određenog studenta
router.get('/:kolegijId/:studentId', isLoggedIn, isStudent, catchAsync(async (req, res) => {

    const { kolegijId, studentId } = req.params;
    const student = await Student.findById(studentId);
    const kolegij = await Kolegij.findById(kolegijId).populate({
        path: 'ispiti',
        populate: {
            path: 'rezultati'
        }
    }).populate('nositelj')
    
    const ispiti = await Ispit.find({ kolegij: kolegij });
    //dohvat svih ispita koje je prijavio odabrani student
    const prijavljeni = await Ispit.find({ 'rezultati.student': studentId }).populate('kolegij').populate('rezultati');
    const neprijavljeni = [{}]
    let pomocna = false;
    
    ispiti.forEach(ispit => {
        pomocna = false
        ispit.rezultati.forEach(r => {
            if (r.student.equals(studentId)) {
                pomocna = true;
            }
        })
        if (!pomocna) neprijavljeni.push(ispit)//ako student nije pronaden unutar rezultata taj ispit se dodaje u niz neprijavljeni koji saljemo u showKolegij
    })

    res.render('student/showKolegij', { kolegij, student, prijavljeni, neprijavljeni });


}))


//putanja za prijavu ispita
router.post('/ispiti/prijava', isLoggedIn, isStudent, catchAsync(async (req, res) => {

    const { studentId, ispitId, kolegijId } = req.body;
    const ispit = await Ispit.findById(ispitId);
    const student = await Student.findById(studentId);
    student.ispiti.push(ispit);
    ispit.rezultati.push({ student: studentId, ocjena: null });
    await ispit.save();
    await student.save();
    res.redirect(`/student/${kolegijId}/${studentId}`)


}));


module.exports = router;